/**************************************************************************************
Author:			David Szybala
Program:		Material_IDs.ms
Date:			01/13/11
Description:	This script will assign material ID modifiers to the selected objects or
material IDs to the selected shaders.
***************************************************************************************/

try (destroyDialog materialIDRollout) catch()

(--put script into local scope
	global materialIDRollout
	local allMaterials = #()
	local materialNames = #()
	local nameOffsets = #()
	local matIDIniFile = getDir #plugcfg + "\\MaterialIDs.ini"
	local materialIDRolloutPos
	
	/***************************************************************************************/
	fn Sort_By_Name elementOne elementTwo =
	(
		case of
		(
			(elementOne.name < elementTwo.name): -1
			(elementOne.name > elementTwo.name): 1
			default: 0
		)--end case
	)--end fn
	
	/***************************************************************************************/
	fn Get_Material_Names =
	(
		materialNames = #() --clear previous names
		materialIDRollout.matNameList.selection = #() --clear selection
		local offsetValue = ""
		
		for i = 1 to nameOffsets.count do
		(
			for x = 1 to nameOffsets[i] do offsetValue += "   "
			materialNames[i] = offsetValue + allMaterials[i].name
			offsetValue = ""
		)--end for
		
		materialIDRollout.matNameList.items = materialNames
	)--end fn
	
	/***************************************************************************************/
	fn Get_Materials =
	(
		allMaterials = #() --clear previous materials
		nameOffsets = #() --clear previous offsets
		
		local currentSubMat = undefined
		local index = 0
		
		for currentMat in sceneMaterials do
			if (superClassOf currentMat) == material do append allMaterials currentMat
			
		qsort allMaterials Sort_By_Name
		
		for current in allMaterials do
		(
			append materialNames current.name
			append nameOffsets 0
		)--end for
		
		for currentMat in allMaterials do
		(
			if (getNumSubMtls currentMat) > 0 do
			(
				for x = 1 to (getNumSubMtls currentMat) do
				(
					currentSubMat = getSubMtl currentMat x
					
					if currentSubMat != undefined do
					(
						index = findItem allMaterials currentMat
						insertItem currentSubMat allMaterials (index + x)
						insertItem (nameOffsets[index] + 1) nameOffsets (index + x)
					)--end if
				)--end for
			)--end if
		)--end for
	)--end fn
	
	/***************************************************************************************/
	fn Set_Sub_Material_ID currentMat =
	(
		local currentSubMat = undefined
		local counter = 1
		
		if (getNumSubMtls currentMat) > 0 do
		(
			for x = 1 to (getNumSubMtls currentMat) do
			(
				currentSubMat = getSubMtl currentMat x
				
				if currentSubMat != undefined do
				(
					if (classOf currentSubMat != Multimaterial) then
					(
						currentSubMat.effectsChannel = counter
					
						counter += 1
						if counter > 15 do counter = 1
					)--end if
					else
						Set_Sub_Material_ID currentSubMat
				)--end if
			)--end for
		)--end if
	)--end fn
	
	/***************************************************************************************/
	fn Refresh_List =
	(
		local theState = false
		local counter = 0
		
		Get_Materials()
		Get_Material_Names()
		
		for currentName in materialNames do
			if currentName[1] != " " do counter += 1
			
		materialIDRollout.totalLbl.text = ("Total: " + (counter as string))
		
		if allMaterials.count > 0 do theState = true
		
		materialIDRollout.assignMatIDBtn.enabled = materialIDRollout.idSpn.enabled = materialIDRollout.assignSubIDBtn.enabled = \
		materialIDRollout.resetIDBtn.enabled = theState
	)--end fn
	
	/***************************************************************************************/
	rollout materialIDRollout "Material IDs"
	(
		groupBox modGrp "Modifier" pos:[5,5] width:200 height:125
			radioButtons idModeRadio "" pos:[15,25] width:75 height:32 labels:#("Constant", "Randomize") columns:1 default:2
			label minLbl "Min:" pos:[110,25] width:25 height:15
			spinner minSpn "" pos:[140,25] width:55 height:15 range:[1,65535,1] type:#integer scale:1
			label maxLbl "Max:" pos:[110,45] width:25 height:15
			spinner maxSpn "" pos:[140,45] width:55 height:15 range:[2,65535,10] type:#integer scale:1
			checkbox selectOnlyChk "Selection Only" pos:[15,65] width:125 height:15 checked:true
			checkbox collapseChk "Collapse Stack" pos:[15,85] width:125 height:15
			button matModBtn "Assign Material Modifier(s)" pos:[15,105] width:180 height:20
			
		groupBox matGrp "Material" pos:[5,135] width:200 height:300
			label totalLbl "Total: 0" pos:[15,155] width:100 height:15
			button updateListBtn "Update" pos:[145,155] width:50 height:15
			MultiListBox matNameList "" pos:[15,175] width:180 height:13
			button assignMatIDBtn "Assign Material ID" pos:[15,360] width:135 height:20 enabled:false
			spinner idSpn "" pos:[160,361] width:35 height:15 range:[0,15,1] type:#integer scale:1 enabled:false
			button assignSubIDBtn "Assign Sub Material IDs" pos:[15,385] width:180 height:20 enabled:false
			button resetIDBtn "Reset All Material IDs" pos:[15,410] width:180 height:20 enabled:false
			
		-------------------------------------------------------------------
		on materialIDRollout open do Refresh_List()
		
		-------------------------------------------------------------------
		on materialIDRollout moved pos do
			setIniSetting matIDIniFile "Dialog" "Position" (pos as string)
			
		-------------------------------------------------------------------	
		on idModeRadio changed theState do
		(
			if theState == 1 then
				maxLbl.enabled = maxSpn.enabled = false
			else
				maxLbl.enabled = maxSpn.enabled = true
		)--end on
		
		-------------------------------------------------------------------
		on updateListBtn pressed do Refresh_List()
		
		-------------------------------------------------------------------
		on matModBtn pressed do
		(
			local currentObjects = #()
			
			max create mode
			
			if selectOnlyChk.checked then
				currentObjects = selection as array
			else
				currentObjects = geometry
				
			for current in currentObjects do
			(
				if idModeRadio.state == 2 then
					addModifier current (Materialmodifier materialID:(random minSpn.value maxSpn.value))
				else
					addModifier current (Materialmodifier materialID:(minSpn.value))
			)--end for
			
			if collapseChk.checked do collapseStack currentObjects
		)--end on
		
		-------------------------------------------------------------------
		on assignMatIDBtn pressed do
		(
			local currentSelection = matNameList.selection as array
			
			for i = 1 to currentSelection.count do
			(
				allMaterials[currentSelection[i]].effectsChannel = idSpn.value
			)--end for
		)--end on
		
		-------------------------------------------------------------------
		on assignSubIDBtn pressed do
		(
			local currentSelection = matNameList.selection as array
			
			for i = 1 to currentSelection.count do Set_Sub_Material_ID allMaterials[currentSelection[i]]
		)--end on
		
		-------------------------------------------------------------------
		on resetIDBtn pressed do for current in allMaterials do current.effectsChannel = 0
	)--end rollout
	materialIDRolloutPos = execute (getIniSetting matIDIniFile "Dialog" "Position")
	if materialIDRolloutPos == OK do materialIDRolloutPos = [100,100]
	if materialIDRolloutPos.x > sysinfo.desktopsize.x - 210 do materialIDRolloutPos.x = sysinfo.desktopsize.x - 215
	if materialIDRolloutPos.y > sysinfo.desktopsize.y - 500 do materialIDRolloutPos.y = sysinfo.desktopsize.y - 500
	if materialIDRolloutPos.x < 0 do materialIDRolloutPos.x = 0
	if materialIDRolloutPos.y < 0 do materialIDRolloutPos.y = 0
	createDialog materialIDRollout 210 440 materialIDRolloutPos.x materialIDRolloutPos.y
)--end local scope